// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2022 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        06/03/2023                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Classe pour stocker les informations aux points             *
 *          d'intégration thermique                                     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef PTINTEGTHERMIINTERNE_H
#define PTINTEGTHERMIINTERNE_H

#include "Tenseur.h"
#include "Vecteur.h"
#include "Temps_CPU_HZpp.h"

/// @addtogroup Groupe_concernant_les_points_integration
///  @{
///


class  PtIntegThermiInterne

{// surcharge de l'operator de lecture
 friend istream & operator >> (istream &, PtIntegThermiInterne &);
 // surcharge de l'operator d'ecriture
 friend ostream & operator << (ostream &, const PtIntegThermiInterne &);

public :
 // CONSTRUCTEURS :
 // contructeur par défaut
 PtIntegThermiInterne();
 // contructeur fonction de la dimension de tenseurs
 PtIntegThermiInterne(int dimtens);
 // contructeur de copie
 PtIntegThermiInterne(const PtIntegThermiInterne& pti);
 
 // DESTRUCTEUR :
 ~PtIntegThermiInterne();
 
 // METHODES PUBLIQUES :
	// Surcharge de l'operateur = 
 PtIntegThermiInterne& operator= ( const PtIntegThermiInterne& pti);
 
 // la température
 double& Temperature() {return temperature;};
 // la température à t
 double& Temperature_t() {return temperature_t;};
 // gradient thermique finale
 CoordonneeB& GradTB() {return gradTB;};
	// vitesse finale du gradient thermique
	CoordonneeB& DgradTB() {return  dgradTB;};
	// variation du gradient thermique entre t et t + delta t
	CoordonneeB& DeltaGradTB() {return  deltaGradTB;};
	// vecteur densité du flux thermique finale
	CoordonneeH& FluxH() {return  fluxH;};
	// vecteur densité du flux thermique en début d'incrément
	CoordonneeH& FluxH_t() {return  fluxH_t;};
 // --- temps cpu
 // tps cpu relatif à la métrique uniquement
 Temps_CPU_HZpp& TpsMetrique() {return tpsMetrique;};
 // temps cumulé relatif à la loi de comportement
 Temps_CPU_HZpp& Tps_cpu_loi_comp() {return  tps_cpu_loi_comp;};

	
	// ---- acces idem en constants 
 // la température
 const double& Temperature_const() const {return temperature;};
 // la température à t
 const double& Temperature_t_const() const {return temperature_t;};
 // gradient thermique finale
 const CoordonneeB &  GradTB_const() const {return gradTB;};
	// vitesse finale du gradient thermique
	const CoordonneeB & DgradTB_const() const {return  dgradTB;};
	// variation du gradient thermique entre t et t + delta t
	const CoordonneeB & DeltaGradTB_const() const {return  deltaGradTB;};
	// densité de flux finale
	const CoordonneeH & FluxH_const() const {return  fluxH;};
	// densité de flux en début d'incrément
	const CoordonneeH & FluxH_t_const() const {return  fluxH_t;};
 // --- temps cpu
 // tps cpu relatif à la métrique uniquement
 const Temps_CPU_HZpp& TpsMetrique_const() const {return tpsMetrique;};
 // temps cumulé relatif à la loi de comportement
 const Temps_CPU_HZpp& Tps_cpu_loi_comp_const() const {return  tps_cpu_loi_comp;};
	
	

	// invariant du gradient thermique
	double&  Norme_gradT() {return  norme_gradT;};
	const double&  Norme_gradT_const() const {return  norme_gradT;};
	// invariant de la vitesse du gradient thermique
	double&  Norme_DGradT() {return  norme_dGradT;};
	const double&  Norme_DGradT_const() const {return  norme_dGradT;};
	// invariant de la densité de flux
	double&  Norme_flux() {return  norme_flux;};
	const double&  Norme_flux_const() const {return  norme_flux;};
	 
 // actualisation des grandeurs actives de t+dt vers t, pour celles qui existent 
 // sous ces deux formes    
 void TdtversT();
 // actualisation des grandeurs actives de t vers tdt, pour celles qui existent 
 // sous ces deux formes          
 void TversTdt();
	 
	//========= méthode particulière pour un passage de l'ordre 2D à 3D des tenseurs et l'inverse ===========
	// plusZero: = true: indique qu'il faut complèter les grandeurs manquantes avec des 0
	//           = false: on ne complète pas
	// il faut que ptintmec comporte des tenseurs d'ordre 2 et this des tenseurs 3D
	void Affectation_2D_a_3D(const PtIntegThermiInterne& ptinther,bool plusZero);
	// l'inverse: comme le conteneur d'arrivée est plus petit, il n'y a pas de complétion
	// il faut que ptintmec comporte des tenseurs d'ordre 3 et this des tenseurs 2D
	void Affectation_3D_a_2D(const PtIntegThermiInterne& ptinther);

	//========= méthode particulière pour un passage de l'ordre 1D à 3D des tenseurs et l'inverse ===========
	// plusZero: = true: indique qu'il faut complèter les grandeurs manquantes avec des 0
	//           = false: on ne complète pas
	// il faut que ptintmec comporte des tenseurs d'ordre 1 et this des tenseurs 3D
	void Affectation_1D_a_3D(const PtIntegThermiInterne& ptinther,bool plusZero);
	// l'inverse: comme le conteneur d'arrivée est plus petit, il n'y a pas de complétion
	// il faut que ptintmec comporte des tenseurs d'ordre 3 et this des tenseurs 1D
	void Affectation_3D_a_1D(const PtIntegThermiInterne& ptinther);

	//============= lecture écriture dans base info ==========
 // cas donne le niveau de la récupération
 // = 1 : on récupère tout
 // = 2 : on récupère uniquement les données variables (supposées comme telles)
	void Lecture_base_info (istream& ent,const int cas);
 // cas donne le niveau de sauvegarde
 // = 1 : on sauvegarde tout
 // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	void Ecriture_base_info(ostream& sort,const int cas);
    		
    
	protected :

      // VARIABLES PROTÉGÉES :    
 
	double  temperature,temperature_t;
 CoordonneeB  gradTB; // gradient thermique finale
	CoordonneeB  dgradTB; // vitesse finale du gradient thermique
	CoordonneeB  deltaGradTB; // variation du gradient thermique entre t et t + delta t
	CoordonneeH  fluxH; // vecteur densité du flux thermique finale
	CoordonneeH  fluxH_t; // vecteur densité du flux thermique en début d'incrément
	
	// ---- les invariants
	double  norme_gradT	; // norme du gradient thermique = invariant
 double  norme_dGradT	; // norme de la vitesse du gradient thermique = invariant
 double  norme_flux	; // norme du vecteur l densité de flux thermique

 // --- temps cpu
 Temps_CPU_HZpp tpsMetrique;       // tps cpu relatif à la métrique uniquement
 Temps_CPU_HZpp tps_cpu_loi_comp;  // temps cumulé relatif à la loi de comportement

 };
 /// @}  // end of group

#endif  
